@@ -18,11 +18,11 @@ This module is a thin layer of abstraction around the library.
 It exposes all core functionality.
 """
 
-from ncclient import capabilities
 from ncclient import operations
 from ncclient import transport
-import six
+import socket
 import logging
+import functools
 
 from ncclient.xml_ import *
 
@@ -47,6 +47,7 @@ OPERATIONS = {
     "kill_session": operations.KillSession,
     "poweroff_machine": operations.PoweroffMachine,
     "reboot_machine": operations.RebootMachine,
+    "rpc": operations.GenericRPC,
 }
 
 """
@@ -56,8 +57,6 @@ subclasses. It is used to lookup operations, e.g. `get_config` is mapped to
 operations to the :class:`Manager` API.
 """
 
-VENDOR_OPERATIONS = {}
-
 
 def make_device_handler(device_params):
     """
@@ -88,40 +87,53 @@ def make_device_handler(device_params):
     return handler_obj
 
 
+def _extract_device_params(kwds):
+    device_params = kwds.pop("device_params", None)
+
+    return device_params
+
+def _extract_manager_params(kwds):
+    manager_params = kwds.pop("manager_params", {})
+
+    # To maintain backward compatibility
+    if 'timeout' not in manager_params and 'timeout' in kwds:
+        manager_params['timeout'] = kwds['timeout']
+    return manager_params
+
+def _extract_nc_params(kwds):
+    nc_params = kwds.pop("nc_params", {})
+
+    return nc_params
+
 def connect_ssh(*args, **kwds):
-    """
-    Initialize a :class:`Manager` over the SSH transport.
+    """Initialize a :class:`Manager` over the SSH transport.
     For documentation of arguments see :meth:`ncclient.transport.SSHSession.connect`.
 
     The underlying :class:`ncclient.transport.SSHSession` is created with
-        :data:`CAPABILITIES`. It is first instructed to
-        :meth:`~ncclient.transport.SSHSession.load_known_hosts` and then
-        all the provided arguments are passed directly to its implementation
-        of :meth:`~ncclient.transport.SSHSession.connect`.
+    :data:`CAPABILITIES`. All the provided arguments are passed directly to its
+    implementation of :meth:`~ncclient.transport.SSHSession.connect`.
 
-    To invoke advanced vendor related operation add device_params =
-        {'name':'<vendor_alias>'} in connection paramerers. For the time,
-        'junos' and 'nexus' are supported for Juniper and Cisco Nexus respectively.
+    To customize the :class:`Manager`, add a `manager_params` dictionary in connection
+    parameters (e.g. `manager_params={'timeout': 60}` for a bigger RPC timeout parameter)
+
+    To invoke advanced vendor related operation add
+    `device_params={'name': '<vendor_alias>'}` in connection parameters. For the time,
+    'junos' and 'nexus' are supported for Juniper and Cisco Nexus respectively.
+
+    A custom device handler can be provided with
+    `device_params={'handler':<handler class>}` in connection parameters.
 
-    A custom device handler can be provided with device_params =
-        {'handler':<handler class>} in connection paramerers.
     """
-    # Extract device parameter dict, if it was passed into this function. Need to
-    # remove it from kwds, since the session.connect() doesn't like extra stuff in
-    # there.
-    if "device_params" in kwds:
-        device_params = kwds["device_params"]
-        del kwds["device_params"]
-    else:
-        device_params = None
+    # Extract device/manager/netconf parameter dictionaries, if they were passed into this function.
+    # Remove them from kwds (which should keep only session.connect() parameters).
+    device_params = _extract_device_params(kwds)
+    manager_params = _extract_manager_params(kwds)
+    nc_params = _extract_nc_params(kwds)
 
     device_handler = make_device_handler(device_params)
     device_handler.add_additional_ssh_connect_params(kwds)
-    global VENDOR_OPERATIONS
-    VENDOR_OPERATIONS.update(device_handler.add_additional_operations())
+    device_handler.add_additional_netconf_params(nc_params)
     session = transport.SSHSession(device_handler)
-    if "hostkey_verify" not in kwds or kwds["hostkey_verify"]:
-        session.load_known_hosts()
 
     try:
        session.connect(*args, **kwds)
@@ -129,26 +141,39 @@ def connect_ssh(*args, **kwds):
         if session.transport:
             session.close()
         raise
-    return Manager(session, device_handler, **kwds)
+    return Manager(session, device_handler, **manager_params)
+
+
+def connect_tls(*args, **kwargs):
+    """Initialize a :class:`Manager` over the TLS transport."""
+    device_params = _extract_device_params(kwargs)
+    manager_params = _extract_manager_params(kwargs)
+    nc_params = _extract_nc_params(kwargs)
+
+    device_handler = make_device_handler(device_params)
+    device_handler.add_additional_netconf_params(nc_params)
+    session = transport.TLSSession(device_handler)
+
+    session.connect(*args, **kwargs)
+
+    return Manager(session, device_handler, **manager_params)
+
 
 def connect_ioproc(*args, **kwds):
-    if "device_params" in kwds:
-        device_params = kwds["device_params"]
-        del kwds["device_params"]
+    device_params = _extract_device_params(kwds)
+    manager_params = _extract_manager_params(kwds)
+
+    if device_params:
         import_string = 'ncclient.transport.third_party.'
         import_string += device_params['name'] + '.ioproc'
         third_party_import = __import__(import_string, fromlist=['IOProc'])
-    else:
-        device_params = None
 
     device_handler = make_device_handler(device_params)
 
-    global VENDOR_OPERATIONS
-    VENDOR_OPERATIONS.update(device_handler.add_additional_operations())
     session = third_party_import.IOProc(device_handler)
     session.connect()
 
-    return Manager(session, device_handler, **kwds)
+    return Manager(session, device_handler, **manager_params)
 
 
 def connect(*args, **kwds):
@@ -161,36 +186,24 @@ def connect(*args, **kwds):
         else:
             return connect_ssh(*args, **kwds)
 
+def call_home(*args, **kwds):
+    host = kwds["host"]
+    port = kwds.get("port",4334)
+    srv_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    srv_socket.bind((host, port))
+    srv_socket.settimeout(10)
+    srv_socket.listen()
 
-class OpExecutor(type):
+    sock, remote_host = srv_socket.accept()
+    logger.info('Callhome connection initiated from remote host {0}'.format(remote_host))
+    kwds['sock'] = sock
+    return connect_ssh(*args, **kwds)
 
-    def __new__(cls, name, bases, attrs):
-        def make_wrapper(op_cls):
-            def wrapper(self, *args, **kwds):
-                return self.execute(op_cls, *args, **kwds)
-            wrapper.__doc__ = op_cls.request.__doc__
-            return wrapper
-        for op_name, op_cls in six.iteritems(OPERATIONS):
-            attrs[op_name] = make_wrapper(op_cls)
-        return super(OpExecutor, cls).__new__(cls, name, bases, attrs)
-
-    def __call__(cls, *args, **kwargs):
-        def make_wrapper(op_cls):
-            def wrapper(self, *args, **kwds):
-                return self.execute(op_cls, *args, **kwds)
-            wrapper.__doc__ = op_cls.request.__doc__
-            return wrapper
-        if VENDOR_OPERATIONS:
-            for op_name, op_cls in six.iteritems(VENDOR_OPERATIONS):
-                setattr(cls, op_name, make_wrapper(op_cls))
-        return super(OpExecutor, cls).__call__(*args, **kwargs)
-
-
-class Manager(six.with_metaclass(OpExecutor, object)):
+class Manager(object):
 
     """
     For details on the expected behavior of the operations and their
-        parameters refer to :rfc:`4741`.
+        parameters refer to :rfc:`6241`.
 
     Manager instances are also context managers so you can use it like this::
 
@@ -208,12 +221,20 @@ class Manager(six.with_metaclass(OpExecutor, object)):
 
    # __metaclass__ = OpExecutor
 
-    def __init__(self, session, device_handler, timeout=30, *args, **kwargs):
+
+    HUGE_TREE_DEFAULT = False
+    """Default for `huge_tree` support for XML parsing of RPC replies (defaults to False)"""
+
+    def __init__(self, session, device_handler, timeout=30):
         self._session = session
         self._async_mode = False
         self._timeout = timeout
         self._raise_mode = operations.RaiseMode.ALL
+        self._huge_tree = self.HUGE_TREE_DEFAULT
         self._device_handler = device_handler
+        self._vendor_operations = {}
+        if device_handler:
+            self._vendor_operations.update(device_handler.add_additional_operations())
 
     def __enter__(self):
         return self
@@ -237,7 +258,8 @@ class Manager(six.with_metaclass(OpExecutor, object)):
                    device_handler=self._device_handler,
                    async_mode=self._async_mode,
                    timeout=self._timeout,
-                   raise_mode=self._raise_mode).request(*args, **kwds)
+                   raise_mode=self._raise_mode,
+                   huge_tree=self._huge_tree).request(*args, **kwds)
 
     def locked(self, target):
         """Returns a context manager for a lock on a datastore, where
@@ -263,16 +285,21 @@ class Manager(six.with_metaclass(OpExecutor, object)):
         raise NotImplementedError
 
     def __getattr__(self, method):
-        """Parse args/kwargs correctly in order to build XML element"""
-        def _missing(*args, **kwargs):
-            m = method.replace('_', '-')
-            root = new_ele(m)
-            if args:
-                for arg in args:
-                    sub_ele(root, arg)
-            r = self.rpc(root)
-            return r
-        return _missing
+        if method in self._vendor_operations:
+            return functools.partial(self.execute, self._vendor_operations[method])
+        elif method in OPERATIONS:
+            return functools.partial(self.execute, OPERATIONS[method])
+        else:
+            """Parse args/kwargs correctly in order to build XML element"""
+            def _missing(*args, **kwargs):
+                m = method.replace('_', '-')
+                root = new_ele(m)
+                if args:
+                    for arg in args:
+                        sub_ele(root, arg)
+                r = self.rpc(root)
+                return r
+            return _missing
 
     def take_notification(self, block=True, timeout=None):
         """Attempt to retrieve one notification from the queue of received
@@ -336,3 +363,13 @@ class Manager(six.with_metaclass(OpExecutor, object)):
     exceptions. Valid values are the constants defined in
     :class:`~ncclient.operations.RaiseMode`.
     The default value is :attr:`~ncclient.operations.RaiseMode.ALL`."""
+
+    @property
+    def huge_tree(self):
+        """Whether `huge_tree` support for XML parsing of RPC replies is enabled (default=False)
+        The default value is configurable through :attr:`~ncclient.manager.Manager.HUGE_TREE_DEFAULT`"""
+        return self._huge_tree
+
+    @huge_tree.setter
+    def huge_tree(self, x):
+        self._huge_tree = x
