@@ -16,7 +16,7 @@
 
 from ncclient.xml_ import *
 
-from rpc import RaiseMode, RPC
+from ncclient.operations.rpc import RaiseMode, RPC
 
 # TODO: parse session-id from a lock-denied error, and raise a tailored exception?
 
